package com.muyu.data.source.service.impl;

import java.sql.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.data.source.domain.*;
import com.muyu.data.source.domain.req.DataSourceSaveReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.data.source.mapper.DataSourceMapper;
import com.muyu.data.source.service.DataSourceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

/**
 * 数据源Service业务层处理
 *
 * @author muyu
 * @date 2024-04-21
 */
@Slf4j
@Service
public class DataSourceServiceImpl extends ServiceImpl<DataSourceMapper, DataSource>  implements DataSourceService {
    @Autowired
    private DataSourceMapper dataSourceMapper;

    /**
     * 查询数据源列表
     *
     * @param dataSource 数据源
     * @return 数据源
     */
    @Override
    public List<DataSource> list(DataSource dataSource) {
        LambdaQueryWrapper<DataSource> queryWrapper = new LambdaQueryWrapper<>();



        if (ObjUtils.notNull(dataSource.getAccessSourceName())){
            queryWrapper.like(DataSource::getAccessSourceName, dataSource.getAccessSourceName());
        }

        if (ObjUtils.notNull(dataSource.getDataSourceSystemName())){
            queryWrapper.like(DataSource::getDataSourceSystemName, dataSource.getDataSourceSystemName());
        }

        if (ObjUtils.notNull(dataSource.getHostAddress())){
            queryWrapper.eq(DataSource::getHostAddress, dataSource.getHostAddress());
        }

        if (ObjUtils.notNull(dataSource.getHostPort())){
            queryWrapper.eq(DataSource::getHostPort, dataSource.getHostPort());
        }

        if (ObjUtils.notNull(dataSource.getDataAccessTypeId())){
            queryWrapper.eq(DataSource::getDataAccessTypeId, dataSource.getDataAccessTypeId());
        }

        if (ObjUtils.notNull(dataSource.getDatabaseName())){
            queryWrapper.like(DataSource::getDatabaseName, dataSource.getDatabaseName());
        }

        if (ObjUtils.notNull(dataSource.getDataConnectionParameter())){
            queryWrapper.eq(DataSource::getDataConnectionParameter, dataSource.getDataConnectionParameter());
        }

        if (ObjUtils.notNull(dataSource.getInitialQuantity())){
            queryWrapper.eq(DataSource::getInitialQuantity, dataSource.getInitialQuantity());
        }

        if (ObjUtils.notNull(dataSource.getMaximumQuantity())){
            queryWrapper.eq(DataSource::getMaximumQuantity, dataSource.getMaximumQuantity());
        }



        if (ObjUtils.notNull(dataSource.getMaximumFrequency())){
            queryWrapper.eq(DataSource::getMaximumFrequency, dataSource.getMaximumFrequency());
        }





        return list(queryWrapper);
    }

    @Override
    public List<AccessType> findAccessType() {
        return dataSourceMapper.findAccessType();
    }

    @Override
    public Result testConnection(DataSourceSaveReq dataSourceQueryReq) {
        String user = dataSourceQueryReq.getDatabaseUserName();
        String password = dataSourceQueryReq.getDatabaseUserPassword();
        String jdbcDriver = "com.mysql.cj.jdbc.Driver";
        String url = "jdbc:mysql://"+dataSourceQueryReq.getHostAddress()+":"+dataSourceQueryReq.getHostPort()+"/"+dataSourceQueryReq.getDatabaseName();
        try {
            Connection con = DriverManager.getConnection(url, user, password);
            if (con!=null){
                return Result.success("连接成功");
            }
            Statement stmt = con.createStatement();

        } catch (SQLException e) {
           log.error("连接失败："+e.getMessage());
            return Result.error("连接失败");
        }

      return Result.success();
    }

    @Override
    public Result synchronization(DataSource dataSource) {

        //初始化容器
        AssetStructure assetStructure = new AssetStructure();
        List<DatabaseTableInformation> list = new ArrayList<>();

                //连接jdbc驱动
                String user=dataSource.getDatabaseUserName();
                String password=dataSource.getDatabaseUserPassword();
                String jdbcDriver="com.mysql.cj.jdbc.Driver";
                String url = "jdbc:mysql://" + dataSource.getHostAddress() + ":" + dataSource.getHostPort() + "/" + dataSource.getDatabaseName();
                //查询条件
                String query="show tables";
        try (Connection con = DriverManager.getConnection(url, user, password)) {
            // 创建 PreparedStatement，并设置参数
            PreparedStatement prepared = con.prepareStatement("SELECT " +
                    " TABLE_NAME AS `name`, " +
                    " TABLE_COMMENT AS `as`, " +
                    " TABLE_ROWS AS `dataTotal` " +
                    " FROM INFORMATION_SCHEMA.TABLES " +
                    " WHERE TABLE_SCHEMA = ?");
            prepared.setString(1, dataSource.getDatabaseName()); // 设置数据库名称参数


            // 执行查询
            ResultSet resultSet = prepared.executeQuery();

            ResultSetMetaData rsmd = resultSet.getMetaData();
            // 5. 处理查询结果
            while (resultSet.next()) {
                String tableName = resultSet.getString("name");
                String tableComment = resultSet.getString("as");
                int tableRows = resultSet.getInt("dataTotal");

                // 创建一个包含数据库表信息的对象
                DatabaseTableInformation databaseTableInformation = DatabaseTableInformation.builder()
                        .name(tableName)
                        .as(tableComment)
                        .dataTotal(tableRows).build();

                // 将对象添加到 ArrayList 中
                list.add(databaseTableInformation);
            }
            //参数复制对象
            assetStructure.setAccessSourceName(dataSource.getAccessSourceName());
            assetStructure.setDatabaseName(dataSource.getDatabaseName());
            assetStructure.setDataSourceSystemName(dataSource.getDataSourceSystemName());
            assetStructure.setDatabaseTableName(rsmd.getTableName(1));
            assetStructure.setDataVolume(rsmd.getColumnDisplaySize(1));
            assetStructure.setDataQuantity(rsmd.getColumnDisplaySize(1));
            if (list.size()==0){
                return Result.error("数据库没有内容");
            }
            //同步到数据库
            dataSourceMapper.addAssets(assetStructure);
            //同步到数据库表信息库
            dataSourceMapper.addDatabaseTableInformation(list,assetStructure.getId());
            List<DataBaseTable> dataBaseTable = new ArrayList<>();
            //列表循环
            for (DatabaseTableInformation databaseTableInformation : list) {
                //条件sql
                PreparedStatement preparedStatement = con.prepareStatement(" SELECT " +
                        "            TABLE_NAME AS 'tableName', " +
                        "                COLUMN_NAME AS 'name', " +
                        "                COLUMN_COMMENT AS 'comment', " +
                        "                CASE WHEN COLUMN_KEY = 'PRI' THEN '是' ELSE '否' END AS 'isPrimaryKey', " +
                        " " +
                        "                CASE " +
                        "                    WHEN DATA_TYPE = 'int' THEN 'Integer' " +
                        "                    WHEN DATA_TYPE = 'varchar' THEN 'String' " +
                        "                    WHEN DATA_TYPE = 'decimal' THEN 'BigDecimal' " +
                        "                    WHEN DATA_TYPE = 'tinyint' AND COLUMN_TYPE = 'tinyint(1)' THEN 'Boolean' " +
                        " " +
                        "                    ELSE DATA_TYPE " +
                        "                    END AS 'type', " +
                        "                DATA_TYPE AS 'mappingType', " +
                        "                COLUMN_TYPE AS 'detailType', " +
                        "                CHARACTER_MAXIMUM_LENGTH AS 'length', " +
                        "                NUMERIC_SCALE AS 'decimalPlaces', " +
                        "                IS_NULLABLE AS 'isNull', " +
                        "                COLUMN_DEFAULT AS 'defaultValue' " +
                        "        FROM " +
                        "            INFORMATION_SCHEMA.COLUMNS " +
                        "        WHERE " +
                        "            TABLE_SCHEMA = ?" +
                        "          AND TABLE_NAME = ?");
                preparedStatement.setString(1, dataSource.getDatabaseName());
                preparedStatement.setString(2, databaseTableInformation.getName());
                //执行查询
                ResultSet executeQuery = preparedStatement.executeQuery();
                ResultSetMetaData metaData = executeQuery.getMetaData();
                // 5. 处理查询结果
                while (executeQuery.next()) {
                    //获得返回值
                    String tableName = executeQuery.getString("tableName");
                    String name = executeQuery.getString("name");
                    String comment = executeQuery.getString("comment");
                    String isPrimaryKey = executeQuery.getString("isPrimaryKey");
                    String type = executeQuery.getString("type");
                    String detailType = executeQuery.getString("detailType");
                    Integer length = executeQuery.getInt("length");
                    Double decimalPlaces = executeQuery.getDouble("decimalPlaces");
                    String isNull = executeQuery.getString("isNull");
                    String defaultValue = executeQuery.getString("defaultValue");
                    //build构造方法
                    DataBaseTable build = DataBaseTable.builder()
                            .name(name)
                            .tableName(tableName)
                            .comment(comment)
                            .decimalPlaces(decimalPlaces)
                            .isPrimaryKey(isPrimaryKey)
                            .type(type)
                            .detailType(detailType)
                            .length(length)
                            .isNull(isNull)
                            .defaultValue(defaultValue)
                            .informationId(databaseTableInformation.getId())
                            .build();
                    //添加到列表
                    dataBaseTable.add(build);
                }
            }
                //添加数据库
                dataSourceMapper.addDataBaseTable(dataBaseTable);
            return Result.success("同步成功");
        } catch (SQLException e) {
            System.out.println("连接失败"+e.getMessage());
            return Result.error("同步失败");
        }

    }



    @Override
    public Result<List<AssetStructure>> findAssetStructure() {
        List<AssetStructure> assetStructureList = dataSourceMapper.assetStructureList();
        return Result.success(assetStructureList);
    }

    @Override
    public Result<List<DatabaseTableInformation>> findInformationById(Integer id) {
        Structure structure = new Structure();
        AssetStructure assetStructure=dataSourceMapper.findAssetsById(id);
        List<DatabaseTableInformation> databaseTableInformationList=dataSourceMapper.findInformationById(id);
        ArrayList<DataBaseTable> dataBaseTableArrayList = new ArrayList<>();
        for (DatabaseTableInformation databaseTableInformation : databaseTableInformationList) {
            List<DataBaseTable> dataBaseTableList=dataSourceMapper.findDataBaseTable(assetStructure.getDatabaseName(),databaseTableInformation.getName());
            dataBaseTableArrayList.addAll(dataBaseTableList);
        }
        System.out.println(dataBaseTableArrayList);
        return Result.success(databaseTableInformationList);
    }

    @Override
    public Result<List<DataBaseTable>> findDataBaseTableById(Integer id) {
        List<DatabaseTableInformation> databaseTableInformation=dataSourceMapper.findDataBaseTableInformationById(id);

        ArrayList<DataBaseTable> dataBaseTables = new ArrayList<>();
        for (DatabaseTableInformation tableInformation : databaseTableInformation) {
            List<DataBaseTable> dataBaseTableList=dataSourceMapper.findDataBaseTableById(tableInformation.getId());
            dataBaseTables.addAll(dataBaseTableList);
        }
        return Result.success(dataBaseTables);
    }

    @Override
    public Result<List<DataBaseTable>> findDataBaseTableByName(String name) {
        List<DataBaseTable> dataBaseTableList=dataSourceMapper.findDataBaseTableByName(name);
        return Result.success(dataBaseTableList);
    }

    @Override
    public Result<DatabaseTableInformation> findDataBaseTable(Long id) {
        DatabaseTableInformation databaseTableInformation=dataSourceMapper.findDataBaseTableBy(id);
        return Result.success(databaseTableInformation);
    }

    @Override
    public Result<List<DatabaseTableInformation>> findDataBaseByAssetId(Integer id) {

        List<DatabaseTableInformation> databaseTableInformationList=dataSourceMapper.findDataBaseTableInformationById(id);
        return Result.success(databaseTableInformationList);
    }

    @Override
    public Result<List<DataBaseTable>> findDataBaseByInformationId(Integer id) {
        List<DataBaseTable> dataBaseTableList=dataSourceMapper.findDataBaseTableById(id);
        return Result.success(dataBaseTableList);
    }

    @Override
    public Result<List<DatabaseTableInformation>> dataBaseTableInformation() {

        List<DatabaseTableInformation> databaseTableInformationList=dataSourceMapper.dataBaseTableInformation();
        return Result.success(databaseTableInformationList);
    }

    @Override
    public Result updateDatabaseTable(DataBaseTable dataBaseTable) {
        Integer integer = dataSourceMapper.updateDatabaseTable(dataBaseTable);
        return Result.success("修改成功");
    }


}
